
import { DataTypes, Model, Optional, Sequelize } from "sequelize";

interface CustomerAttributes {
  id: number;
  firstName: string;
  lastName: string;
  email: string;
}
export interface CustomerInput extends Optional<CustomerAttributes, "id"> { }
export interface CustomerOutput extends Required<CustomerAttributes> { }
class Customer extends Model<CustomerAttributes> implements CustomerAttributes {
  public id!: number;
  public firstName!: string;
  public lastName: string;
  public email: string;

  public readonly createdAt!: Date;
  public readonly updatedAt!: Date;
}

export const initCustomer = async (sequelizeConnection: Sequelize) => {

Customer.init(
  {
    id: {
      type: DataTypes.INTEGER,
      allowNull: false,
      primaryKey: true,
    },
    firstName: {
      type: new DataTypes.STRING(),
      allowNull: false,
    },
    lastName: {
      type: new DataTypes.STRING(),
      allowNull: false,
    },
    email: {
      type: new DataTypes.STRING(),
      allowNull: false,
      unique: true,
    },
  },
  {
    tableName: "customer",
    sequelize: sequelizeConnection,
  }
);
}

export const create = async (payload: Customer): Promise<CustomerOutput> => {
  return await Customer.create(payload);
};

export const update = async (
  id: number,
  payload: Partial<CustomerInput>
): Promise<CustomerOutput> => {
  const customer = await Customer.findByPk(id);
  if (!customer) {
    throw new Error("not found");
  }
  return await (customer as Customer).update(payload);
};

export const getById = async (id: number): Promise<CustomerInput> => {
  const customer = await Customer.findByPk(id);
  if (!customer) {
    throw new Error("not found");
  }
  return customer;
};

const deleteById = async (id: number): Promise<boolean> => {
  const deletedCustomerCount = await Customer.destroy({
    where: { id },
  });
  return !!deletedCustomerCount;
};
export const getAll = async (): Promise<CustomerOutput[]> => {
  return Customer.findAll();
};
export const CustomerService = {
  create,
  update,
  getById,
  deleteById,
  getAll,
};

export default Customer;
